defmodule Mnemonics.Mixfile do
  use Mix.Project

  def project do
    [
      app: :mnemonics,
      deps: deps(),
      description: "Read only data store for Elixir: fast, concurrently, for large data & hot reloadable.",
      elixir: "~> 1.5",
      elixirc_paths: elixirc_paths(Mix.env),
      package: package(),
      start_permanent: Mix.env == :prod,
      version: "0.1.1",

      # Docs
      docs: [
        main: "readme",
        extras: ["README.md"],
      ],
      homepage_url: "https://github.com/ne-sachirou/mnemonics",
      name: "Mnemonics",
      source_url: "https://github.com/ne-sachirou/mnemonics",
    ]
  end

  def application do
    [
      extra_applications: [:logger],
      mod: {Mnemonics.Application, []}
    ]
  end

  defp deps do
    [
      {:ex_doc, "~> 0.16", only: :dev, runtime: false},
      {:fastglobal, "~> 1.0"},
      {:inner_cotton, github: "ne-sachirou/inner_cotton", only: [:dev, :test], runtime: false},
    ]
  end

  defp elixirc_paths(:test), do: ["lib", "test/support"]
  defp elixirc_paths(_), do: ["lib"]

  def package do
    [
      files: ["LICENSE", "README.md", "mix.exs", "lib"],
      licenses: ["GPL-3.0"],
      links: %{
        "GitHub": "https://github.com/ne-sachirou/mnemonics",
      },
      name: :mnemonics,
      maintainers: ["ne_Sachirou <utakata.c4se@gmail.com>"],
    ]
  end
end
